/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks.stone;

import java.util.Random;

import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.PropertyInteger;
import net.minecraft.block.state.BlockFaceShape;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.MathHelper;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.common.IPlantable;

import net.dries007.tfc.api.types.Rock;

public class BlockFarmlandTFC extends BlockRockVariant
{
    public static final int MAX_MOISTURE = 15;
    public static final PropertyInteger MOISTURE = PropertyInteger.func_177719_a("moisture", 0, MAX_MOISTURE);
    public static final int[] TINT = new int[] {
        0xffffffff,
        0xfff7f7f7,
        0xffefefef,
        0xffe7e7e7,
        0xffdfdfdf,
        0xffd7d7d7,
        0xffcfcfcf,
        0xffc7c7c7,
        0xffbfbfbf,
        0xffb7b7b7,
        0xffafafaf,
        0xffa7a7a7,
        0xff9f9f9f,
        0xff979797,
        0xff8f8f8f,
        0xff878787,
    };
    protected static final AxisAlignedBB FARMLAND_AABB = new AxisAlignedBB(0.0D, 0.0D, 0.0D, 1.0D, 0.9375D, 1.0D);
    protected static final AxisAlignedBB FLIPPED_AABB = new AxisAlignedBB(0.0D, 0.9375D, 0.0D, 1.0D, 1.0D, 1.0D);

    public BlockFarmlandTFC(Rock.Type type, Rock rock)
    {
        super(type, rock);
        func_180632_j(field_176227_L.func_177621_b().func_177226_a(MOISTURE, 1)); // 1 is default so it doesn't instantly turn back to dirt
        func_149675_a(true);
        func_149713_g(255);
    }

    @Override
    @SuppressWarnings("deprecation")
    public IBlockState func_176203_a(int meta)
    {
        return this.func_176223_P().func_177226_a(MOISTURE, meta);
    }

    @Override
    public int func_176201_c(IBlockState state)
    {
        return state.func_177229_b(MOISTURE);
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_149686_d(IBlockState state)
    {
        return false;
    }

    @Override
    @SuppressWarnings("deprecation")
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess source, BlockPos pos)
    {
        return FARMLAND_AABB;
    }

    @Override
    @SuppressWarnings("deprecation")
    public BlockFaceShape func_193383_a(IBlockAccess worldIn, IBlockState state, BlockPos pos, EnumFacing face)
    {
        return face == EnumFacing.DOWN ? BlockFaceShape.SOLID : BlockFaceShape.UNDEFINED;
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_149662_c(IBlockState state)
    {
        return false;
    }

    @Override
    public void func_180658_a(World worldIn, BlockPos pos, Entity entityIn, float fallDistance)
    {
        if (!worldIn.field_72995_K && entityIn.canTrample(worldIn, this, pos, fallDistance)) // Forge: Move logic to Entity#canTrample
        {
            turnToDirt(worldIn, pos);
        }
        super.func_180658_a(worldIn, pos, entityIn, fallDistance);
    }

    @Override
    protected BlockStateContainer func_180661_e()
    {
        return new BlockStateContainer(this, MOISTURE);
    }

    @Override
    public ItemStack getPickBlock(IBlockState state, RayTraceResult target, World world, BlockPos pos, EntityPlayer player)
    {
        return new ItemStack(Item.func_150898_a(this));
    }

    @Override
    public void func_180650_b(World world, BlockPos pos, IBlockState state, Random rand)
    {
        int current = state.func_177229_b(MOISTURE);
        int target = world.func_175727_C(pos.func_177984_a()) ? MAX_MOISTURE : getWaterScore(world, pos);

        if (current < target)
        {
            if (current < MAX_MOISTURE) world.func_180501_a(pos, state.func_177226_a(MOISTURE, current + 1), 2);
        }
        else if (current > target || target == 0)
        {
            if (current > 0) world.func_180501_a(pos, state.func_177226_a(MOISTURE, current - 1), 2);
            else if (!hasCrops(world, pos)) turnToDirt(world, pos);
        }

        super.func_180650_b(world, pos, state, rand);
    }

    @Override
    public Item func_180660_a(IBlockState state, Random rand, int fortune)
    {
        return Item.func_150898_a(get(rock, Rock.Type.DIRT));
    }

    public int getWaterScore(IBlockAccess world, BlockPos pos)
    {
        final int hRange = 7;
        float score = 0;
        for (BlockPos.MutableBlockPos i : BlockPos.func_177975_b(pos.func_177982_a(-hRange, -1, -hRange), pos.func_177982_a(hRange, 2, hRange)))
        {
            BlockPos diff = i.func_177973_b(pos);
            float hDist = MathHelper.func_76129_c(diff.func_177958_n() * diff.func_177958_n() + diff.func_177952_p() * diff.func_177952_p());
            if (hDist > hRange) continue;
            if (world.func_180495_p(i).func_185904_a() != Material.field_151586_h) continue;
            score += ((hRange - hDist) / (float) hRange);
        }
        return score > 1 ? MAX_MOISTURE : Math.round(score * MAX_MOISTURE);
    }

    public void turnToDirt(World world, BlockPos pos)
    {
        world.func_175656_a(pos, get(rock, Rock.Type.DIRT).func_176223_P());
        AxisAlignedBB axisalignedbb = FLIPPED_AABB.func_186670_a(pos);
        for (Entity entity : world.func_72839_b(null, axisalignedbb))
        {
            double d0 = Math.min(axisalignedbb.field_72337_e - axisalignedbb.field_72338_b, axisalignedbb.field_72337_e - entity.func_174813_aQ().field_72338_b);
            entity.func_70634_a(entity.field_70165_t, entity.field_70163_u + d0 + 0.001D, entity.field_70161_v);
        }
    }

    private boolean hasCrops(World worldIn, BlockPos pos)
    {
        Block block = worldIn.func_180495_p(pos.func_177984_a()).func_177230_c();
        return block instanceof IPlantable && canSustainPlant(worldIn.func_180495_p(pos), worldIn, pos, EnumFacing.UP, (IPlantable) block);
    }

    //todo: onBlockAdded turn back if solid above
    //todo: neighborChanged turn back if solid above

}
