/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.items;

import java.util.Arrays;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.material.Material;
import net.minecraft.block.state.IBlockState;
import net.minecraft.client.renderer.block.model.ModelResourceLocation;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.EnumAction;
import net.minecraft.item.ItemStack;
import net.minecraft.util.*;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.world.World;
import net.minecraftforge.client.model.ModelLoader;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import mcp.MethodsReturnNonnullByDefault;
import net.dries007.tfc.api.util.Size;
import net.dries007.tfc.api.util.Weight;

import static net.dries007.tfc.Constants.MOD_ID;

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
public class ItemGoldPan extends ItemTFC
{
    private static final String[] TYPES = new String[] {"empty", "sand", "gravel", "clay", "dirt"};

    @SuppressWarnings("ConstantConditions")
    @SideOnly(Side.CLIENT)
    public static void registerModels()
    {
        for (int meta = 0; meta < TYPES.length; meta++)
            ModelLoader.setCustomModelResourceLocation(ItemsTFC.GOLDPAN, meta, new ModelResourceLocation(MOD_ID + ":goldpan/" + TYPES[meta]));
        ModelLoader.registerItemVariants(ItemsTFC.GOLDPAN, Arrays.stream(TYPES).map(e -> new ResourceLocation(MOD_ID, "goldpan/" + e)).toArray(ResourceLocation[]::new));
    }

    public ItemGoldPan()
    {
        func_77656_e(0);
        func_77625_d(1);
        setNoRepair();
        func_77627_a(true);
    }

    @Override
    public ActionResult<ItemStack> func_77659_a(World worldIn, EntityPlayer playerIn, EnumHand handIn)
    {
        //todo: move to public EnumActionResult onItemUse(EntityPlayer player, World worldIn, BlockPos pos, EnumHand hand, EnumFacing facing, float hitX, float hitY, float hitZ)
        ItemStack stack = playerIn.func_184586_b(handIn);
        if (worldIn.field_72995_K) return new ActionResult<>(EnumActionResult.PASS, stack);
        if (handIn != EnumHand.MAIN_HAND) return new ActionResult<>(EnumActionResult.PASS, stack);
        if (canPan(worldIn, playerIn) == null) return new ActionResult<>(EnumActionResult.FAIL, stack);
        playerIn.func_184598_c(handIn);
        return new ActionResult<>(EnumActionResult.SUCCESS, stack);
    }

    @Override
    public String func_77667_c(ItemStack stack)
    {
        return super.func_77667_c(stack) + "." + TYPES[stack.func_77952_i()];
    }

    @Override
    public EnumAction func_77661_b(ItemStack stack)
    {
        return EnumAction.BOW;
    }

    @Override
    public int func_77626_a(ItemStack stack)
    {
        return 72;
    }

    @Override
    public void func_150895_a(CreativeTabs tab, NonNullList<ItemStack> items)
    {
        if (!func_194125_a(tab)) return;

        for (int meta = 0; meta < TYPES.length; meta++)
            items.add(new ItemStack(this, 1, meta));
    }

    @Override
    public void onUsingTick(ItemStack stack, EntityLivingBase entityLivingBase, int countLeft)
    {
        if (!(entityLivingBase instanceof EntityPlayer)) return;
        final EntityPlayer player = ((EntityPlayer) entityLivingBase);
        final RayTraceResult result = canPan(player.field_70170_p, player);
        if (result == null)
        {
            player.func_184602_cy();
            return;
        }
        final int total = func_77626_a(stack);
        final int count = total - countLeft;
        final BlockPos pos = result.func_178782_a().func_177982_a(0, 1, 0);
        final World world = player.field_70170_p;
        final float chance = world.field_73012_v.nextFloat();

        //todo: this is a copypaste from firestarter, it needs to pan, not start fires.
    }

    @Override
    public Size getSize(ItemStack stack)
    {
        return Size.SMALL;
    }

    @Override
    public Weight getWeight(ItemStack stack)
    {
        return Weight.LIGHT;
    }

    @Override
    public boolean canStack(ItemStack stack)
    {
        return false;
    }

    @Nullable
    private RayTraceResult canPan(World world, EntityPlayer player)
    {
        RayTraceResult result = func_77621_a(world, player, true);
        //noinspection ConstantConditions
        if (result == null) return null;
        if (result.field_72313_a != RayTraceResult.Type.BLOCK) return null;
        BlockPos pos = result.func_178782_a();
        final IBlockState current = player.field_70170_p.func_180495_p(pos);
        pos = pos.func_177982_a(0, 1, 0);
        if (world.func_180495_p(pos).func_185904_a() != Material.field_151586_h) return null;
        return result;
    }
}
